<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Customer Group Catalog for Magento 2
 */

namespace Amasty\Groupcat\Plugin\Block\Navigation;

use Amasty\Groupcat\Api\Data\RuleInterface;
use Amasty\Groupcat\Model\OptionSource\Rule\PriceAction;
use Amasty\Groupcat\Model\ResourceModel\Rule;
use Magento\CatalogSearch\Model\Layer\Filter\Price;
use Magento\Customer\Model\Session;
use Magento\Framework\Stdlib\DateTime\TimezoneInterface;
use Magento\LayeredNavigation\Block\Navigation as MagentoNavigation;
use Magento\Store\Model\StoreManagerInterface;

class HidePriceFilter
{
    /**
     * @var Session
     */
    private $session;

    /**
     * @var Rule
     */
    private $rule;

    /**
     * @var TimezoneInterface
     */
    private $localeDate;

    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    public function __construct(
        Session $session,
        Rule $rule,
        TimezoneInterface $localeDate,
        StoreManagerInterface $storeManager
    ) {
        $this->session = $session;
        $this->storeManager = $storeManager;
        $this->rule = $rule;
        $this->localeDate = $localeDate;
    }

    public function afterGetFilters(MagentoNavigation $subject, array $filterList): array
    {
        $customerGroupId = $this->session->isLoggedIn()
            ? $this->session->getCustomer()->getGroupId()
            : 0;
        $storeId = $this->storeManager->getStore()->getId();
        $dateTimestamp = $this->localeDate->scopeTimeStamp($storeId);
        $query = $this->rule->getActiveMatchesSelect($dateTimestamp, $storeId, $customerGroupId);
        $activeRules = $this->rule->getConnection()->fetchRow($query);

        if ($activeRules && $activeRules[RuleInterface::PRICE_ACTION] != PriceAction::SHOW && !empty($filterList)) {
            foreach ($filterList as $key => $filter) {
                if ($filter instanceof Price) {
                    unset($filterList[$key]);
                }
            }
        }

        return $filterList;
    }
}
