<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Customer Group Catalog for Magento 2
 */

namespace Amasty\Groupcat\Plugin\Quote\Model\Quote;

use Amasty\Groupcat\Helper\Data;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Catalog\Model\ResourceModel\Product\CollectionFactory as ProductCollectionFactory;
use Magento\Framework\Registry;
use Magento\Quote\Model\Quote;

class FixLoadQuoteWithRestrictedItems
{
    /**
     * @var bool
     */
    private $skipRestrict = false;

    /**
     * @var Data
     */
    private $helper;

    /**
     * @var Registry
     */
    private $coreRegistry;

    /**
     * @var ProductCollectionFactory
     */
    private $productCollectionFactory;

    public function __construct(
        Data $helper,
        Registry $coreRegistry,
        ProductCollectionFactory $productCollectionFactory
    ) {
        $this->helper = $helper;
        $this->coreRegistry = $coreRegistry;
        $this->productCollectionFactory = $productCollectionFactory;
    }

    public function beforeGetAllVisibleItems(Quote $subject): void
    {
        $this->coreRegistry->register('amasty_ignore_product_filter_for_quote', true);
    }

    public function afterGetAllVisibleItems(Quote $subject, array $result): array
    {
        $this->coreRegistry->unregister('amasty_ignore_product_filter_for_quote');

        if ($this->skipRestrict && !empty($result)) {
            return $result;
        }

        $itemsProductIds = array_map(function ($item) {
            return $item->getProductId();
        }, $result);

        $productCollection = $this->productCollectionFactory->create()
            ->setStoreId($subject->getStoreId())
            ->addFieldToFilter('entity_id', ['in' => $itemsProductIds]);
        $existProductIds = $productCollection->getAllIds();
        $triggerRecollect = false;

        foreach ($result as $key => $item) {
            if (!in_array($item->getProductId(), $existProductIds)) {
                $subject->deleteItem($item);
                unset($result[$key]);
                $triggerRecollect = true;
            }
        }

        if ($triggerRecollect) {
            $subject->setTotalsCollectedFlag(false)
                ->collectTotals()
                ->save();
        }

        return $result;
    }
    public function beforeRemoveItem(Quote $subject): void
    {
        $this->skipRestrict = true;
    }

    public function afterRemoveItem(Quote $subject): void
    {
        $this->skipRestrict = false;
    }
}
